import React, { useRef, useCallback, useEffect, useState, useMemo } from "react";
import * as d3 from "d3";
import useResizeObserver from "use-resize-observer";
import throttle from "lodash/throttle";
var RESIZE_THROTTLE_TIME = 500; // ms

function ForceDirectedGraph(_ref) {
  var data = _ref.data,
    containerHeight = _ref.containerHeight,
    onClickNode = _ref.onClickNode,
    onMouseOverNode = _ref.onMouseOverNode,
    onMouseOutNode = _ref.onMouseOutNode,
    _ref$nodeCharge = _ref.nodeCharge,
    nodeCharge = _ref$nodeCharge === void 0 ? -250 : _ref$nodeCharge;
  var containerRef = useRef(null);
  var _useResizeObserver = useResizeObserver({
      ref: containerRef
    }),
    containerWidth = _useResizeObserver.width;
  var _useRef = useRef(d3.forceSimulation().force("link", d3.forceLink().distance(150) // Minimum distance between linked nodes
    .id(function (d) {
      return d.id;
    })).force("charge", d3.forceManyBody().strength(nodeCharge)) // Force between nodes
    ),
    simulation = _useRef.current;
  var svgRef = useRef(d3.create("svg").attr('display', 'block'));
  var linkRef = useRef(null);
  var nodeRef = useRef(null);
  var labelRef = useRef(null);
  var zoom = useMemo(function () {
    var zoomed = function zoomed() {
      linkRef.current.attr("transform", d3.event.transform);
      nodeRef.current.attr("transform", d3.event.transform);
      labelRef.current.attr("transform", d3.event.transform);
    };
    return d3.zoom().scaleExtent([1 / 8, 8]).on("zoom", zoomed);
  }, []);
  var update = useCallback(function (_ref2) {
    var nodes = _ref2.nodes,
      links = _ref2.links;
    // Make a shallow copy to protect against mutation, while
    // recycling old nodes to preserve position and velocity.
    var old = new Map(nodeRef.current.data().map(function (d) {
      return [d.id, d];
    }));
    nodes = nodes.map(function (d) {
      return Object.assign(old.get(d.id) || {}, d);
    });
    links = links.map(function (d) {
      return Object.assign({}, d);
    });
    nodeRef.current = nodeRef.current.data(nodes).join("circle").attr("r", function (d) {
      var _d$size;
      return (_d$size = d.size) !== null && _d$size !== void 0 ? _d$size : 12;
    }).attr("fill", function (d) {
      return color(d.colorIndex);
    }).attr("stroke-width", function (d) {
      return d.selected ? 2 : 0;
    }).attr("stroke", function (d) {
      return d.selected ? color(3) : '#FFF';
    }).on('click', onClickNode).on('mouseover', onMouseOverNode).on('mouseout', onMouseOutNode).call(drag(simulation));
    labelRef.current = labelRef.current.data(nodes).join("text").attr("dx", function (d) {
      var _d$size2;
      return ((_d$size2 = d.size) !== null && _d$size2 !== void 0 ? _d$size2 : 12) + 2;
    }).attr("dy", ".35em").attr("font-size", 14).text(function (d) {
      return d.label;
    });
    linkRef.current = linkRef.current.data(links).join("line");
    // .attr('marker-end','url(#arrowhead)'); // Add arrow to links

    simulation.nodes(nodes);
    simulation.force("link").links(links);
    simulation.alpha(1).restart();
    var transform = d3.zoomTransform(svgRef.current.node());
    linkRef.current.attr("transform", transform);
    nodeRef.current.attr("transform", transform);
    labelRef.current.attr("transform", transform);
  }, [onClickNode]);
  useEffect(function () {
    // Set d3 SVG group references on mount
    // SVG group for links
    linkRef.current = svgRef.current.append("g").attr("stroke", "grey").attr("stroke-opacity", 0.6).attr("stroke-width", 1).selectAll("line");
    // SVG group for nodes
    nodeRef.current = svgRef.current.append("g").selectAll("circle");
    // SVG group for labels
    labelRef.current = svgRef.current.append("g").selectAll("text");

    // Simulation tick handler for setting positions of SVG elements
    var onTick = function onTick() {
      linkRef.current.attr("x1", function (d) {
        return d.source.x;
      }).attr("y1", function (d) {
        return d.source.y;
      }).attr("x2", function (d) {
        return d.target.x;
      }).attr("y2", function (d) {
        return d.target.y;
      });
      nodeRef.current.attr("cx", function (d) {
        return d.x;
      }).attr("cy", function (d) {
        return d.y;
      });
      labelRef.current.attr("x", function (d) {
        return d.x;
      }).attr("y", function (d) {
        return d.y;
      });
    };

    // Existing listener is replaced on registering new one
    simulation.on("tick", onTick);

    // Append the d3 root element to container div
    containerRef.current.append(svgRef.current.node());
  }, []);
  useEffect(function () {
    svgRef.current.call(zoom);
  }, [zoom]);
  var throttledUpdateViewBox = useMemo(function () {
    return throttle(function (width, height) {
      svgRef.current.attr("viewBox", [0, 0, width, height]);
      simulation.force("center", d3.forceCenter(width / 2, height / 2));
      zoom.extent([[0, 0], [width, height]]);
    }, RESIZE_THROTTLE_TIME);
  }, [zoom]);
  useEffect(function () {
    if (!containerWidth) {
      return;
    }
    throttledUpdateViewBox(containerWidth, containerHeight);
  }, [containerWidth, containerHeight, throttledUpdateViewBox]);
  useEffect(function () {
    update(data);
  }, [data, update]);
  return /*#__PURE__*/React.createElement("div", {
    ref: containerRef
  });
}
export default ForceDirectedGraph;
var color = function color(n) {
  return d3.schemeCategory10[n];
};
var drag = function drag(simulation) {
  function dragstarted(d) {
    if (!d3.event.active) simulation.alphaTarget(0.3).restart();
    d.fx = d.x;
    d.fy = d.y;
  }
  function dragged(d) {
    d.fx = d3.event.x;
    d.fy = d3.event.y;
  }
  function dragended(d) {
    if (!d3.event.active) simulation.alphaTarget(0);
    d.fx = null;
    d.fy = null;
  }
  return d3.drag().on("start", dragstarted).on("drag", dragged).on("end", dragended);
};

// SVG for arrowheads
// svg.append('defs')
//   .append('marker')
//   .attr("id",'arrowhead')
//   .attr('viewBox','-0 -5 10 10') //the bound of the SVG viewport for the current SVG fragment. defines a coordinate system 10 wide and 10 high starting on (0,-5)
//   .attr('refX',26.5) // x coordinate for the reference point of the marker. If circle is bigger, this need to be bigger.
//   .attr('refY',0)
//   .attr('orient','auto')
//   .attr('markerWidth',5)
//   .attr('markerHeight',5)
//   .attr('xoverflow','visible')
//   .append('svg:path')
//   .attr('d', 'M 0,-5 L 10 ,0 L 0,5')
//   .attr('fill', '#999')
//   .style('stroke','none');